pro auto_newt_raph_data, time, theta, dtheta, rho, drho, EL, elfix, ELerr = ELerr,showplot=showplot,nskip=nskip;, toler=toler

; Same as auto_newt_raph.pro, except enter data as time,pa,dpa,sep,dsep
; rather than as a data file (PA and dPA already in radians).
; 
; /showplot = show plots and print results to screen
; toler: set tolerance for chi2 agreement
;
;
; Compute the orbital elements through a Newton-Raphson technique.
;
; This program should work directly with the data points (rho,theta,t), not 
; the fitting points. 
;
; The program requires an initial guess that can be calculated from using
; the points from the fitting curves.
;
; Equations:  
; Initial estimates (P0,T0,e0,A0,B0,F0,G0) and a set of observations 
; (ti,rhoi,thetai) are known.
; The eccentric anomalies Ei can be found from: 
;	u(ti - T) = Ei - esin(Ei)
;	where u = 360/P
; True anomaly:
;	tan(nu/2) = E - esin(E)
; Radius vector:
;	r = a(1-e^2)/(1 + ecos(nu))
; The fitted (rho_i,theta_i) positions are determined from the orbital elements
; 	tan(theta - Omega) = tan(nu + omega)*cosi
;	rho = r*cos(nu + omega)*sec(theta - Omega)
;
; Procedure:
; Minimize chi squared between data point positions and fitted positions:
;	chi^2 = sum[(rhodata - rhoi)^2/sigmarho^2 
;		    + (thetadata - thetai)^2/sigmatheta^2]
; Take partial derivatives of chi^2 with respect to the orbital elements,
; (P,T,e,a,i,Omega,omega) and set to zero. Solve for the value of the orbital 
; elements that minimize chi^2.
;
; Since the partial derivatives are non-linear, cannot solve for the orbital
; elements that minimize chi^2 analytically.
;
; In turn, use a Newton-Raphson technique to converge upon the solution.
; Replace (rhoi,thetai) in the chi^2 equation with the Taylor series 
; approximation,
; 	x = x| + dx/dP|(P-P0) + dx/dT|(T-T0) + dx/de|(e-e0) 
;	       + dx/da|(a-Aa) + dx/di|(i-i0) + dx/dW|(W-W0) + dx/dw|(w-w0)
; where x=(rho,theta) and the | represents the value evaluated at for the
; initial estimates for the orbital elements. Now, rhoi and thetai are linear 
; in (P,T,e,a,i,W,w), so the partial derivatives can easily be taken and solved
; by setting up a matrix and using Cramer's method.
;
; Paramers:
;	Period:	 period (P)
;	Tperi: 	 time of periastron passage (T)
;	ecc:	 eccentricity (e)
;	major:	 semi-major axis (in mas) (a)
;	inc:	 inclination (i)
;	Omega:	 position angle of node (W)
;	omega:	 angle between node and periastron (w)
;	ti:	 time of observation
;	rho_d:	 separation - data
;	theta_d: postion angle - data
;	rho_f:	 sep - fit (determined from orbital elements for time of obs.)
;	theta_f: PA - fit (determined from orbital elements for time of obs.)
;
; INPUT:
;	data_file: file containing data points in the format:
;		      time  theta  rho
;		      where theta is the position angle and rho the separation
;		      (enter file name using quotes)
;	num: number of data points contained in data_file
;	
; Prompted inputs:
;	initial estimates for all the orbital elements: P,T,e,a,i,Omega,omega
;
; Began 14 May 2002
; Modified 11 Nov 2003: 
;       This is the current version of NEWT_RAPH.PRO
;	(v1.1 and v1.2 are older versions)
; 	- restructured program into separate subroutines
; 	- add option of holding elements fixed during iterations
; 13 January 2005:
;       Fix bugs
;       - if solving for only one element, don't use cramer.pro to solve eqn's
;       - fix 360 degree roll overs in (PAdata - PAfit)
;       Save version v4 as backup
; 25 March 2016:
;       Allow for sep,PA to be fit independently so that micrometer 
;       measurements where only sep or only PA are given can be
;       included in the fit.  Assume errors in sep,PA to be 0.0 when 
;       no measurement exists.   
;       - Previous version saved as auto_newt_raph_data_v1.pro


; If keyword showplot is set, then show orbit plots on screen
; keyword_set returns True (1) if showplot is defined, False (0) if undefined
ans_plot = keyword_set(showplot)

; Check if we are outputting the array of parameter errors
ans_err = keyword_set(ELerr)

; Do we need to end loop if solution is not found?
ans_skip = keyword_set(nskip)
if (ans_skip eq 0) then nskip=1d6

;; If keyword showplot is set, then show orbit plots on screen
;ans_tol = keyword_set(toler)
;if(ans_tol) then ctol = toler else ctol = 0.0

num = n_elements(time)

; convert data points to x and y coordinates

xarr = rho * cos(theta)	; x coordinate
yarr = rho * sin(theta)	; y coordinate

; propagate errors in sep and PA to x and y:

dx = sqrt(cos(theta)^2*drho^2 + rho^2*sin(theta)^2*dtheta^2)
dy = sqrt(sin(theta)^2*drho^2 + rho^2*cos(theta)^2*dtheta^2)

; set initial values

period = EL(0)
Tperi = EL(1)
ecc = EL(2)
major = EL(3)
inc = EL(4)
W_cap = EL(5)
w_low = EL(6)

;convert i, Omega, omega to radians
inc = inc*!dpi/180
W_cap = W_cap*!dpi/180
w_low = w_low*!dpi/180

k=0
for i=0, 6 do if (elfix(i) eq 0) then k=k+1

nEl = n_elements(elfix)
mfit = nEl - k 		; number of elements to improve

; determine indices of elements for which to vary
elvar = where(elfix ne 0)

EL = [period,Tperi,ecc,major,inc,W_cap,w_low]
ELadj = EL
elLabel = ['P','T','e','a','i','Omega','omega']

; Figure out which sep,PA measurements are defined
; If there is only one of the pair measured
; (i.e., for micrometer measurements where the sep,PA
; values are measured independently), then the error
; for the non-existent measure should be 0.0.
ipa = where(dtheta ne 0.0, numpa)
isep = where(drho ne 0.0, numsep)
iboth = where((dtheta ne 0.0) and (drho ne 0.0), nboth)

; plot model and data of initial guess

tnum=1000.0
tstep = period/tnum
tmin = Tperi

tarr = findgen(tnum)*tstep + tmin

calc_vbfit, EL, tarr, theta_mod, rho_mod

xmod = rho_mod * cos(theta_mod)	; x coordinate
ymod = rho_mod * sin(theta_mod)	; y coordinate

; plot best-fit orbit

ans_cont = ' ' 
if (ans_plot) then begin
   plot,-yarr(iboth),xarr(iboth),psym=6
   oplot,-ymod(iboth),xmod(iboth)
   ;print,'Hit enter to continue'
   ;read,ans_cont
endif

count = 0

delta_chi = 1.0		; set initially to begin loop

lambda = 0.001   ; Marquardt method to ensure convergence
lflag = 0

; ***** Begin while loop ***** 


while((delta_chi gt 0.001) and (count lt nskip)) do begin

    ; Determine errors in orbital elements
    ; set up matrices for error determination
    ; Invert matrix through Gauss Jordan elimination (Numerical Recipes in C)

    ; set up covariance matrix and column matrix
    ; alpha = dblarr(7,7)
    ; beta = dblarr(7)

    calc_deriv_vb, EL, elfix, mfit, time, theta, rho, dtheta, drho, theta_f, rho_f, alpha, beta

    ; Determine chi squared

    ; convert data points to x and y coordinates

    xfit = rho_f * cos(theta_f)	; x coordinate
    yfit = rho_f * sin(theta_f)	; y coordinate
			
    ;chi2old = total((xarr-xfit)^2/dx^2 + (yarr-yfit)^2/dy^2)

    diff_theta = theta - theta_f

    ; account for any wrap-arounds from 360 deg to 0 deg

    for k=0, num-1 do begin
        if (diff_theta(k) ge !dpi) then $
          diff_theta(k) = diff_theta(k) - 2*!dpi
        if (diff_theta(k) le -!dpi) then $
          diff_theta(k) = 2*!dpi + diff_theta(k)	
    endfor

    ;chi2old = total((diff_theta)^2/dtheta^2 + (rho - rho_f)^2/drho^2)
    chi2old = total((diff_theta(ipa))^2/dtheta(ipa)^2) + $
              total((rho(isep) - rho_f(isep))^2/drho(isep)^2)

    invmat = invert(alpha, stat, /double)

    if (ans_plot) then print,"stat (0 successful):",stat

    ;determine errors:

    ELerr = dblarr(mfit)

    for i=0, mfit-1 do ELerr(i) = sqrt(invmat(i,i))

    ; adjust alpha matrix by Marquardt parameter lambda

    for i=0, mfit-1 do alpha(i,i) = alpha(i,i)*(1.0 + lambda)

    if (mfit eq 1) then begin
        delta_el = dblarr(1)
        delta_el(0) = beta/alpha
    endif else begin
       ; Avoid singular matrix error
       if (stat eq 0) then delta_el = cramer(alpha,beta) $ ; adjustments 
       else delta_el = dblarr(mfit)
    endelse

    ; adjust orbital parameters
    
    for i=0, mfit-1 do ELadj(elvar(i)) = EL(elvar(i)) + delta_el(i)

    if (ans_plot) then begin
    k=0
    print,"Orbital Element     Adjustment"
    for i=0, 3 do begin
        if (elfix(i) eq 0) then begin
            print, elLabel(i), ': ', El(i)
        endif else begin
            print, elLabel(i), ': ', El(i), delta_el(k)
            k=k+1
        endelse
    endfor
    for i=4, nEl-1 do begin
         if (elfix(i) eq 0) then begin
             print, elLabel(i), ': ', El(i)*180/!dpi
         endif else begin
             print, elLabel(i), ': ', El(i)*180/!dpi, delta_el(k)*180/!dpi
             k = k+1
         endelse
    endfor

    print,"Chi2 of current solution:",chi2old

    plot,-yarr(iboth),xarr(iboth),psym=6
    oplot,-yfit(iboth),xfit(iboth),psym=7
    ;for l=0, num-1 do oplot,[-yarr(l),-yfit(l)],[xarr(l),xfit(l)]
    for l=0, nboth-1 do oplot,[-yarr(iboth(l)),-yfit(iboth(l))],[xarr(iboth(l)),xfit(iboth(l))]

    endif

    ;if (count eq 0) then stop,'Type .cont to continue'

    calc_deriv_vb, ELadj, elfix, mfit, time, theta, rho, dtheta, drho, theta_f, rho_f, alpha, beta

    ; Determine chi squared

    ; convert data points to x and y coordinates

    xfit = rho_f * cos(theta_f)	; x coordinate
    yfit = rho_f * sin(theta_f)	; y coordinate
			
    ;chi2new = total((xarr-xfit)^2/dx^2 + (yarr-yfit)^2/dy^2)

    diff_theta = theta - theta_f

    ; account for any wrap-arounds from 360 deg to 0 deg

    for k=0, num-1 do begin
        if (diff_theta(k) ge !dpi) then $
          diff_theta(k) = diff_theta(k) - 2*!dpi
        if (diff_theta(k) le -!dpi) then $
          diff_theta(k) = 2*!dpi + diff_theta(k)	
    endfor

    ;chi2new = total((diff_theta)^2/dtheta^2 + (rho - rho_f)^2/drho^2)
    chi2new = total((diff_theta(ipa))^2/dtheta(ipa)^2) + $
              total((rho(isep) - rho_f(isep))^2/drho(isep)^2)

    if (ans_plot) then print,"chi2 of next modification:",chi2new

    if (chi2new le (chi2old)) then begin
        EL = ELadj
	chi2 = chi2new
        delta_chi = chi2old - chi2new
        lambda = lambda/10.0
        if (ans_plot) then print, 'lambda', lambda
    endif else begin
	if (ans_plot) then  begin
           print, "next adjustment does not improve chi2"
           print, "increasing lambda by 10.0"
        endif
        lambda = lambda*10.0
        if (ans_plot) then print, 'lambda', lambda
        delta_chi = 1.0 ; prevent exit when lambda is increased
    endelse

    count = count+1

    ; do not exit unless lambda is less than 0.001

;;;;;    if (lambda gt 0.001) and (delta_chi lt 0.001) then delta_chi = 1.0
;    print,'lambda',lambda
;    print,'delta_chi',delta_chi

endwhile


lambda = 0.0

if (ans_plot) then print, "Final values:"

period = EL(0)
Tperi = EL(1)
ecc = EL(2)
major =  EL(3)
inc = EL(4)
W_cap = EL(5)
w_low = EL(6)

calc_deriv_vb, EL, elfix, mfit, time, theta, rho, dtheta, drho, theta_f, rho_f, alpha, beta

invmat = invert(alpha, stat, /double)

;print,'Covariance Matrix'
;print,invmat

;determine errors:

diff_theta = theta - theta_f

; account for any wrap-arounds from 360 deg to 0 deg

for k=0, num-1 do begin
if (diff_theta(k) ge !dpi) then $
	diff_theta(k) = diff_theta(k) - 2*!dpi
if (diff_theta(k) le -!dpi) then $
	diff_theta(k) = 2*!dpi + diff_theta(k)	
endfor

;chi2rt = total((diff_theta)^2/dtheta^2 + (rho - rho_f)^2/drho^2)
chi2rt = total((diff_theta(ipa))^2/dtheta(ipa)^2) + $
         total((rho(isep) - rho_f(isep))^2/drho(isep)^2)

; convert data points to x and y coordinates
xfit = rho_f * cos(theta_f)	; x coordinate
yfit = rho_f * sin(theta_f)	; y coordinate
if (numpa eq numsep) then chi2xy = total((xarr-xfit)^2/dx^2 + (yarr-yfit)^2/dy^2)

; degrees of freedom
; dof = 2d*num - mfit
dof = numsep + numpa - mfit

; reduced chi squared:
chi2red = chi2rt/dof

ELerr = dblarr(mfit)
for i=0,mfit-1 do ELerr(i) = sqrt(chi2red)*sqrt(invmat(i,i))

if (ans_plot) then begin
k=0
print,"Orbital Element     Error"
for i=0, 3 do begin
    if (elfix(i) eq 0) then begin
        print, elLabel(i), ': ', El(i)
    endif else begin
        print, elLabel(i), ': ', El(i), ELerr(k)
        k=k+1
    endelse
endfor
for i=4, nEl-1 do begin
     if (elfix(i) eq 0) then begin
         print, elLabel(i), ': ', El(i)*180/!dpi
     endif else begin
         print, elLabel(i), ': ', El(i)*180/!dpi, ELerr(k)*180/!dpi
         k = k+1
     endelse
endfor
print, "chi2(PA,sep):", chi2rt
print, "chi2red:", chi2red
if (numpa eq numsep) then print, "chi2(x,y):", chi2xy

print,'Number of iterations:',count
;print,'Final chi2:',chi2

print,EL(0),EL(1),EL(2),EL(3),EL(4)*180/!dpi,EL(5)*180/!dpi,EL(6)*180/!dpi

; plot best-fit orbit and data

tnum=1000.0
tstep = period/tnum
tmin = Tperi

tarr = findgen(tnum)*tstep + tmin

calc_vbfit, EL, tarr, theta_mod, rho_mod

xmod = rho_mod * cos(theta_mod)	; x coordinate
ymod = rho_mod * sin(theta_mod)	; y coordinate

; plot data and best-fit positions
plot,-yarr(iboth),xarr(iboth),psym=6
oplot,-yfit(iboth),xfit(iboth),psym=7
;for l=0, num-1 do oplot,[-yarr(l),-yfit(l)],[xarr(l),xfit(l)]
for l=0, nboth-1 do oplot,[-yarr(iboth(l)),-yfit(iboth(l))],[xarr(iboth(l)),xfit(iboth(l))]

; plot best-fit orbit
oplot,-ymod,xmod

endif


;re-set i,Omega,omega to degrees for exit

for i=4,6 do EL(i)=EL(i)*180.0/!dpi

if (ans_err) then for i=4,6 do ELerr(i)=ELerr(i)*180.0/!dpi

end
