pro ephemeris_sb1

; Compute SB1 velocity at specified times
;
; The program requires an initial guess that can be calculated from using
; the points from the fitting curves.
;
; Calls the following routines:
;     calc_deriv_SB1.pro
;     calc_deriv_SB2.pro
;     calc_Ei.pro
;     solve_trans.pro
;     calc_sb2fit.pro - for plotting model orbits
;
; Equations:  
; Initial estimates (P0,T0,e0,a1sini|0,a2sini|0,omega0,Vsys0) and  
; a set of observations (ti,V1i,V2i) are known.
; The eccentric anomalies Ei can be found from: 
;	u(ti - T) = Ei - esin(Ei)
;	where u = 360/P
; True anomaly:
;	tan(nu/2) = E - esin(E)
; Projected semi-major axes (a1sini, a2sini):
;       a1sini = (P/2pi)*K1*(1 - e^2)^1/2
;       a2sini = (P/2pi)*K2*(1 - e^2)^1/2
; Velocity amplitudes (K1, K2):
;       K = (2pi/P)(asini/(1 - e^2)^1/2)
; The fitted velocities (V1_i, V2_i) are determined from the orbital elements:
;       V1 = K1*[e*cos(omega) + cos(nu + omega)] + Vsys
;       V2 = K2*[e*cos(omega) + cos(nu + omega)] + Vsys
;
; Procedure:
; Minimize chi squared between measured velocities and fitted velocities:
;	chi^2 = sum[(V1data - V1fit)^2/sigmaV1^2 
;		    + (V2data - V2fit)^2/sigmaV2^2]
; where the sums run over all data points
; Take partial derivatives of chi^2 with respect to the orbital elements,
; (P,T,e,a1sini,a2sini,omega,Vsys) and set to zero. Solve for the value of
; the orbital elements that minimize chi^2.
;
; Since the equations for V1 and V2 are non-linear, cannot solve for 
; the orbital elements that minimize chi^2 analytically.
;
; In turn, use a Newton-Raphson technique to converge upon the solution.
; Replace (V1fit,V2fit) in the chi^2 equation with the Taylor series 
; approximation,
; 	x = x| + dx/dP|(P-P0) + dx/dT|(T-T0) + dx/de|(e-e0) + dx/dw|(w-w0)
;	       + dx/d(a1sini)|(a1sini - a1sini|0) 
;              + dx/d(a2sini)|(a2sini - a2sini|0)
;              + dx/d(Vsys)|(Vsys - Vsys0)
; where x=(V1,V2) and the | represents the value evaluated for the
; initial estimates of the orbital elements. Now, V1fit and V2fit 
; are linear in (P,T,e,a1sini,a2sini,w,Vsys), so the partial derivatives
; can easily be taken and solved by setting up a matrix and using 
; Cramer's method.
;
; Parameters:
;	Period:	 period (P)
;	Tperi: 	 time of periastron passage (T)
;	ecc:	 eccentricity (e)
;       K1:      velocity amplitude of primary (in km/s) (K1)
;       K2:      velocity amplitude of secondary (in km/s) (K2)
;	a1sini:	 projected semi-major axis of primary orbit (in km) (a1sini)
;	a2sini:	 projected semi-major axis of secondary orbit (in km) (a2sini)
;	omega:	 angle between node and periastron (w)
;	ti:	 time of observation
;	V1_d:	 velocity of primary - measured data
;	V2_d:    velocity of secondary - measured data
;	V1_f:	 V1 - fit (determined from orbital elements for time of obs.)
;	V2_f:    V2 - fit (determined from orbital elements for time of obs.)
;       Vsys:    systemic velocity
;
; INPUT:
;	data_file: file containing data points in the format:
;		      time  V1  errV1  V2  errV2
;		      where V1 is the radial velocity of the primary,
;                     V2 is the radial velocity of the secondary,
;                     and errV1 and errV2 are the measurement errors
;		      (enter file name using quotes)
;	num: number of data points contained in data_file
;
; Prompted inputs:
;	initial estimates for all the orbital elements: 
;             P, T, e, a1sini, a2sini, omega, Vsys
;       vector identifying which orbital elements to solve for
;             enter 0 to hold element fixed, 1 to vary)
;             ex.   1 1 1 1 1 1 1 -> solves for all
;                   0 0 0 1 0 0 0 -> solves only for a1 of primary
; OUTPUT:
;	best fit values of orbital elements: P,T,e,a1sini,a2sini,omega,Vsys
;       and corresponding errors
;
; Notes on units:
;    P, T, and time should be in same units (days or years)
;    K1, K2, V1, V2 should be in same units (km/s)
; Notes on units:
;    Velocities should be in km/s
;    P, T, and time should be in same units (days or years)
;    asini can be in any units (conversion to meters is prompted for)
;        -> program converts to km when computed fit-velocities    
;
; Began 23 June 2005
;
; Version 1: Orignal copy
; Version 2: 
;    - Include Marquadt Method of adjusting diagonal elements of
;      the alpha matrix to ensure convergence. 5 July 2005
;    - Fixed non-convergence problems   7 July 2005
;        + do not remove P wrap around effects from dV/dP
;          leave (t-T)/P in full form; net effect is that adjustments
;          smaller
;        + fixed primary/secondary indices in setting dV1/dVsys and dV2/dVsys
;    - Designed for SB2's only!!!
; Version 3: 7 July 2005
;    - Add SB1 + SB2 joint solutions
; 
; 20 October 2005: Modification of fit_velocity_sb2sb1.pro
;
; 26 March 2007 Save unmodified program as version 1
; 26 March 2007 Fit SB2 parameters in terms of K1 and K2
;               (as opposed to a1sini and a2sini)
;



!P.charsize=1.2
!P.charthick=2.0
!P.thick=2.0
!X.thick=2.0
!Y.thick=2.0
frac=1.4
symthick=2.0

;!P.charsize=1.5
;!P.charthick=4.0
;!P.thick=5.0
;!X.thick=5.0
;!Y.thick=5.0
;frac=1.5        ; scale factor for adjusting size of plotting symbols
;symthick=5.0    ; thickness for symbol lines

;!P.charsize=1.25
;!P.charthick=4.0
;!P.thick=5.0
;!X.thick=5.0
;!Y.thick=5.0
;frac=1.2
;symthick=5.0


; Set color table
;  red=1, green=2, blue=3

tvlct, [255,0,0], [0,255,0], [0,0,255], 1

; Read in data points from data files

temp1 = 0d
temp2 = 0d
temp3 = 0d	; temporary variables to read data from file
temp4 = 0d
temp5 = 0d


; Obtain values for P,T,e,a1sini,a2sini,omega,Vsys

period = 0d
Tperi = 0d
ecc = 0d
K1 = 0d
omega = 0d
Vsys = 0d

print,"Enter P,T,e,K1,omega,Vsys:"
read,period,Tperi,ecc,K1,omega,Vsys

print,"Initial P,T,e,K1,omega,Vsys:"
print,period,Tperi,ecc,K1,omega,Vsys

;convert omega to radians
omega = omega*!dpi/180

;ansyd=' '
;print,'Is period in years or days (y/d)?'
;read,ansyd
;
;if (ansyd eq 'y') then y_d = 365.25
;if (ansyd eq 'd') then y_d = 1.0
;
;; conversion factor for y/d to seconds
;; (need to convert period to sections when computing velocities)
;sec = 1.0*y_d*24.0*60.0*60.0
;
;meters = 0.0
;print,'How many meters are in semi-major axis units?'
;print,'(1 Gm = 1d9 m)'
;read,meters
;
;km = meters/1d3

EL = [period,Tperi,ecc,K1,omega,Vsys]
EL_sb1 = [period,Tperi,ecc,K1,omega,Vsys]

ELadj = EL
ELadj_sb1 = EL_sb1


elLabel = ['P','T','e','K1','omega','Vsys']

;; compute phase from time of periastron passage
;phase_sb1 = (time_sb1 - Tperi)/period
;; reduce phase to -1.0 to 1.0
;phase_sb1 = phase_sb1 - fix(phase_sb1)
;; keep phase between 0.0 to 1.0
;for i=0, sb1_num-1 do if (phase_sb1(i) lt 0.0) $
;  then phase_sb1(i) = phase_sb1(i) + 1.0


time_sb1 = 0.0d

print,'Enter time for which you would like to compute the SB1 radial velocity:'
read,time_sb1

calc_sb1fit, EL_sb1, time_sb1, V1mod

print,'Date   V1   V1-Vsys'
print,format='(a3,f9.1,f6.2,f6.2)','V1: ',time_sb1,V1mod(0), V1mod(0) - Vsys



; Compute velocity curve over 2 periods

tnum = 2000.0
tmin = time_sb1 - 2.5*period
tmax = time_sb1 + 0.5*period
tstep = (tmax-tmin)/tnum

tarr = findgen(tnum)*tstep + tmin

;phase_mod = (tarr-Tperi)/period
;phase_mod = phase_mod - fix(phase_mod)
;for i=0, tnum-1 do if (phase_mod(i) lt 0.0) $
;  then phase_mod(i) = phase_mod(i) + 1.0

calc_sb1fit, EL_sb1, tarr, V1arr

vmax = 40;max(V1arr)
vmin = -40;min(V1arr)


set_plot, 'ps'
device, /Color, Bits_per_pixel=8, filename='temp.ps';,xsize=17,ysize=15

plot,tarr,V1arr,yrange=[vmin,vmax],xrange =[min(tarr),max(tarr)], $
  xtitle = 'Reduced Julian Date (days)', ytitle = 'Velocity (km/s)', xstyle=1,ystyle=1,xtickformat='(i6)';'(f7.1)'

oplot,[min(tarr),max(tarr)],[Vsys,Vsys],Linestyle=2

; For eps Aur:
;Local midnight = 2000 Jan 1,  8 hr UT, or JD 2451544.833
;Local midnight = 1949 Jan 2,  8 hr UT, or JD 2432918.833
;Local midnight = 2049 Jan 2,  8 hr UT, or JD 2469443.833

yr_mark = [32918.0d,51544.0d,69443.0d]
yr_lab = ['1950','2000','2050']
yr_num = n_elements(yr_mark)

for i=0, yr_num-1 do oplot,[yr_mark(i),yr_mark(i)],[Vmin,Vmax],linestyle=1
for i=0, yr_num-1 do xyouts,yr_mark(i),Vmax+1.0,yr_lab(i),alignment=0.5

oplot,[time_sb1,time_sb1],[Vmin,Vmax],linestyle=3

;for i=0, yr_num-1 do oplot,[yr_mark(i),yr_mark(i)],[Vmax,Vmax-2],linestyle=1
;for i=0, yr_num-1 do xyouts,yr_mark(i),Vmax-4,yr_lab(i),alignment=0.5


device, /close
set_plot, 'x'



; Compute velocity curve over 1 phase

tnum = 10000.0
tmin = Tperi - period*0.1
tmax = Tperi + period + period*0.1
tstep = (tmax-tmin)/tnum

tarr = findgen(tnum)*tstep + tmin

phase_mod = (tarr-Tperi)/period
;phase_mod = phase_mod - fix(phase_mod)
;for i=0, tnum-1 do if (phase_mod(i) lt 0.0) $
;  then phase_mod(i) = phase_mod(i) + 1.0

calc_sb1fit, EL_sb1, tarr, V1arr

vmax = max(V1arr)
vmin = min(V1arr)

vind = where(V1arr eq vmax)
print, V1arr(vind), tarr(Vind), phase_mod(vind)


set_plot, 'ps'
device, /Color, Bits_per_pixel=8, filename='temp_phase.ps';,xsize=17,ysize=15

plot,phase_mod,V1arr,yrange=[vmin,vmax],xrange =[-0.1,1.1], $
  xtitle = 'Phase', ytitle = 'Velocity (km/s)', xstyle=1

oplot,[-0.1,1.1],[Vsys,Vsys],Linestyle=2

device, /close
set_plot, 'x'


;for i=0, sb1_num-1 do  print,time_sb1(i),V1_sb1(i),phase_sb1(i)

;; since model is plotted as solid line, it will suffice just to
;; connect data point directly to solid lne (without plotting fit positions)
;for l=0, sb2_num-1 do oplot,[phase_sb2(l),phase_sb2(l)],[V1_sb2(l),V1_f_sb2(l)]
;for l=0, sb2_num-1 do oplot,[phase_sb2(l),phase_sb2(l)],[V2_sb2(l),V2_f_sb2(l)]
;for l=0, sb1_num-1 do oplot,[phase_sb1(l),phase_sb1(l)],[V1_sb1(l),V1_f_sb1(l)]


;free_lun,lun

end

;
;
;

pro plot_erry, x, y, dy, N, color_ind

; plot error bars associated with each data point
; Input
;	x: array of x values
;	dx: array of error in x values
;	y: array of y values
;	dy: erray of error in y values
;	N: number of data points
;       color_ind: color index

for i=0, N-1 do begin

;	oplot, [x(i)-dx(i), x(i)+dx(i)], [y(i), y(i)]
	oplot, [x(i), x(i)], [y(i)-dy(i), y(i)+dy(i)],color=color_ind

endfor

end
