pro plot_velocity_sb2sb1, sb2_file, sb2_num, sb1_file, sb1_num

; Plots SB1 and SB2 orbital fits simultaneously!
;
; The program requires an initial guess that can be calculated from using
; the points from the fitting curves.
;
; Calls the following routines:
;     calc_deriv_SB1.pro
;     calc_deriv_SB2.pro
;     calc_Ei.pro
;     solve_trans.pro
;     calc_sb2fit.pro - for plotting model orbits
;
; Equations:  
; Initial estimates (P0,T0,e0,a1sini|0,a2sini|0,omega0,Vsys0) and  
; a set of observations (ti,V1i,V2i) are known.
; The eccentric anomalies Ei can be found from: 
;	u(ti - T) = Ei - esin(Ei)
;	where u = 360/P
; True anomaly:
;	tan(nu/2) = E - esin(E)
; Projected semi-major axes (a1sini, a2sini):
;       a1sini = (P/2pi)*K1*(1 - e^2)^1/2
;       a2sini = (P/2pi)*K2*(1 - e^2)^1/2
; Velocity amplitudes (K1, K2):
;       K = (2pi/P)(asini/(1 - e^2)^1/2)
; The fitted velocities (V1_i, V2_i) are determined from the orbital elements:
;       V1 = K1*[e*cos(omega) + cos(nu + omega)] + Vsys
;       V2 = K2*[e*cos(omega) + cos(nu + omega)] + Vsys
;
; Procedure:
; Minimize chi squared between measured velocities and fitted velocities:
;	chi^2 = sum[(V1data - V1fit)^2/sigmaV1^2 
;		    + (V2data - V2fit)^2/sigmaV2^2]
; where the sums run over all data points
; Take partial derivatives of chi^2 with respect to the orbital elements,
; (P,T,e,a1sini,a2sini,omega,Vsys) and set to zero. Solve for the value of
; the orbital elements that minimize chi^2.
;
; Since the equations for V1 and V2 are non-linear, cannot solve for 
; the orbital elements that minimize chi^2 analytically.
;
; In turn, use a Newton-Raphson technique to converge upon the solution.
; Replace (V1fit,V2fit) in the chi^2 equation with the Taylor series 
; approximation,
; 	x = x| + dx/dP|(P-P0) + dx/dT|(T-T0) + dx/de|(e-e0) + dx/dw|(w-w0)
;	       + dx/d(a1sini)|(a1sini - a1sini|0) 
;              + dx/d(a2sini)|(a2sini - a2sini|0)
;              + dx/d(Vsys)|(Vsys - Vsys0)
; where x=(V1,V2) and the | represents the value evaluated for the
; initial estimates of the orbital elements. Now, V1fit and V2fit 
; are linear in (P,T,e,a1sini,a2sini,w,Vsys), so the partial derivatives
; can easily be taken and solved by setting up a matrix and using 
; Cramer's method.
;
; Parameters:
;	Period:	 period (P)
;	Tperi: 	 time of periastron passage (T)
;	ecc:	 eccentricity (e)
;       K1:      velocity amplitude of primary (in km/s) (K1)
;       K2:      velocity amplitude of secondary (in km/s) (K2)
;	a1sini:	 projected semi-major axis of primary orbit (in km) (a1sini)
;	a2sini:	 projected semi-major axis of secondary orbit (in km) (a2sini)
;	omega:	 angle between node and periastron (w)
;	ti:	 time of observation
;	V1_d:	 velocity of primary - measured data
;	V2_d:    velocity of secondary - measured data
;	V1_f:	 V1 - fit (determined from orbital elements for time of obs.)
;	V2_f:    V2 - fit (determined from orbital elements for time of obs.)
;       Vsys:    systemic velocity
;
; INPUT:
;	data_file: file containing data points in the format:
;		      time  V1  errV1  V2  errV2
;		      where V1 is the radial velocity of the primary,
;                     V2 is the radial velocity of the secondary,
;                     and errV1 and errV2 are the measurement errors
;		      (enter file name using quotes)
;	num: number of data points contained in data_file
;
; Prompted inputs:
;	initial estimates for all the orbital elements: 
;             P, T, e, a1sini, a2sini, omega, Vsys
;       vector identifying which orbital elements to solve for
;             enter 0 to hold element fixed, 1 to vary)
;             ex.   1 1 1 1 1 1 1 -> solves for all
;                   0 0 0 1 0 0 0 -> solves only for a1 of primary
; OUTPUT:
;	best fit values of orbital elements: P,T,e,a1sini,a2sini,omega,Vsys
;       and corresponding errors
;
; Notes on units:
;    P, T, and time should be in same units (days or years)
;    K1, K2, V1, V2 should be in same units (km/s)
; Notes on units:
;    Velocities should be in km/s
;    P, T, and time should be in same units (days or years)
;    asini can be in any units (conversion to meters is prompted for)
;        -> program converts to km when computed fit-velocities    
;
; Began 23 June 2005
;
; Version 1: Orignal copy
; Version 2: 
;    - Include Marquadt Method of adjusting diagonal elements of
;      the alpha matrix to ensure convergence. 5 July 2005
;    - Fixed non-convergence problems   7 July 2005
;        + do not remove P wrap around effects from dV/dP
;          leave (t-T)/P in full form; net effect is that adjustments
;          smaller
;        + fixed primary/secondary indices in setting dV1/dVsys and dV2/dVsys
;    - Designed for SB2's only!!!
; Version 3: 7 July 2005
;    - Add SB1 + SB2 joint solutions
; 
; 20 October 2005: Modification of fit_velocity_sb2sb1.pro
;
; 26 March 2007 Save unmodified program as version 1
; 26 March 2007 Fit SB2 parameters in terms of K1 and K2
;               (as opposed to a1sini and a2sini)
;



!P.font = 0
!P.charsize=1.0
!P.charthick=2.0
!P.thick=2.0
!X.thick=2.0
!Y.thick=2.0
frac=1.0        ; scale factor for adjusting size of plotting symbols
frac = 0.6
frac = 0.8
symthick=2.0    ; thickness for symbol lines
lthick = 2

;!P.font = 0
;!P.charsize=1.5
;!P.charthick=4.0
;!P.thick=5.0
;!X.thick=5.0
;!Y.thick=5.0
;frac=1.5        ; scale factor for adjusting size of plotting symbols
;symthick=5.0    ; thickness for symbol lines

;!P.charsize=1.25
;!P.charthick=4.0
;!P.thick=5.0
;!X.thick=5.0
;!Y.thick=5.0
;frac=1.2
;symthick=5.0


; Set color table
;  red=1, green=2, blue=3

tvlct, [255,0,0], [0,255,0], [0,0,255], 1

; Read in data points from data files

temp1 = 0d
temp2 = 0d
temp3 = 0d	; temporary variables to read data from file
temp4 = 0d
temp5 = 0d

time_sb2 = dblarr(sb2_num)
V1_sb2 = dblarr(sb2_num)
V2_sb2 = dblarr(sb2_num)
dV1_sb2 = dblarr(sb2_num)
dV2_sb2 = dblarr(sb2_num)

time_sb1 = dblarr(sb1_num)
V1_sb1 = dblarr(sb1_num)
dV1_sb1 = dblarr(sb1_num)

; Read SB2 measurements

openr,lun,sb2_file,/get_lun

for i=0, sb2_num-1 do begin

	readf, lun, temp1, temp2, temp3, temp4, temp5

	time_sb2(i) = temp1
	V1_sb2(i) = temp2
	dV1_sb2(i) = temp3
	V2_sb2(i) = temp4	
	dV2_sb2(i) = temp5
	
endfor

close,lun

; Read SB1 measurements

openr,lun,sb1_file,/get_lun

for i=0, sb1_num-1 do begin

	readf, lun, temp1, temp2, temp3

	time_sb1(i) = temp1
	V1_sb1(i) = temp2
	dV1_sb1(i) = temp3
	
endfor

close,lun

totnum = sb2_num + sb1_num
time = dblarr(totnum)

for i=0, sb2_num-1 do time(i) = time_sb2(i)
for i=sb2_num, totnum-1 do time(i) = time_sb1(i-sb2_num)

; Check to see if there are any 0 weight observations

izero = where((dV1_sb2 eq 0.0) or (dV1_sb2 eq 0.0), num_zero)

if (num_zero gt 0) then begin
   ; Save 0 weight data
   time_sb2_zero = time_sb2(izero)
   V1_sb2_zero = V1_sb2(izero)
   V2_sb2_zero = V2_sb2(izero)
   dV1_sb2_zero = dV1_sb2(izero)
   dV2_sb2_zero = dV2_sb2(izero)
   ; Extract good data
   igood = where((dV1_sb2 ne 0.0) or (dV1_sb2 ne 0.0),sb2_num)
   time_sb2 = time_sb2(igood)
   V1_sb2  = V1_sb2(igood) 
   dV1_sb2 = dV1_sb2(igood)
   V2_sb2  = V2_sb2(igood) 	
   dV2_sb2 = dV2_sb2(igood)
endif

; Obtain values for P,T,e,a1sini,a2sini,omega,Vsys

period = 0d
Tperi = 0d
ecc = 0d
K1 = 0d
K2 = 0d
omega = 0d
Vsys = 0d

print,"Enter P,T,e,K1,K2,omega,Vsys:"
read,period,Tperi,ecc,K1,K2,omega,Vsys

print,"Initial P,T,e,K1,K2,omega,Vsys:"
print,period,Tperi,ecc,K1,K2,omega,Vsys

;convert omega to radians
omega = omega*!dpi/180

;ansyd=' '
;print,'Is period in years or days (y/d)?'
;read,ansyd
;
;if (ansyd eq 'y') then y_d = 365.25
;if (ansyd eq 'd') then y_d = 1.0
;
;; conversion factor for y/d to seconds
;; (need to convert period to sections when computing velocities)
;sec = 1.0*y_d*24.0*60.0*60.0
;
;meters = 0.0
;print,'How many meters are in semi-major axis units?'
;print,'(1 Gm = 1d9 m)'
;read,meters
;
;km = meters/1d3

EL = [period,Tperi,ecc,K1,K2,omega,Vsys]
EL_sb2 = [period,Tperi,ecc,K1,K2,omega,Vsys]
EL_sb1 = [period,Tperi,ecc,K1,omega,Vsys]

ELadj = EL
ELadj_sb2 = EL_sb2
ELadj_sb1 = EL_sb1


elLabel = ['P','T','e','K1','K2','omega','Vsys']

calc_sb2fit, EL_sb2, time_sb2, V1fit, V2fit

; Calculate the chi2:

chi2 = total((V1_sb2 - V1fit)^2/dV1_sb2^2 + (V2_sb2 - V2fit)^2/dV2_sb2^2)

; degrees of freedom
dof = 2.0*sb2_num - 7

; reduced chi squared:
chi2red = chi2/dof

print,'chi2:        ',chi2
print,'Reduced chi2:',chi2red


;; Determine the velocity amplitudes
;K1 = (2.0*!dpi/(period*sec))*(a1sini*km/sqrt(1 - ecc^2))
;K2 = (2.0*!dpi/(period*sec))*(a2sini*km/sqrt(1 - ecc^2))
;print,'K1',K1
;print,'K2',K2

; compute phase from time of periastron passage
phase_sb2 = (time_sb2 - Tperi)/period

; reduce phase to -1.0 to 1.0
phase_sb2 = phase_sb2 - fix(phase_sb2)

; keep phase between 0.0 to 1.0
for i=0, sb2_num-1 do if (phase_sb2(i) lt 0.0) $
  then phase_sb2(i) = phase_sb2(i) + 1.0

; compute phase from time of periastron passage
phase_sb1 = (time_sb1 - Tperi)/period
phase_sb1 = phase_sb1 - fix(phase_sb1)
for i=0, sb1_num-1 do if (phase_sb1(i) lt 0.0) $
  then phase_sb1(i) = phase_sb1(i) + 1.0

if (num_zero gt 0) then begin
   phase_sb2_zero = (time_sb2_zero - Tperi)/period
   phase_sb2_zero = phase_sb2_zero - fix(phase_sb2_zero)
   for i=0, num_zero-1 do if (phase_sb2_zero(i) lt 0.0) $
   then phase_sb2_zero(i) = phase_sb2_zero(i) + 1.0
endif

; plot model and data of initial guess

tnum = 1000.0
tmin = Tperi - period*0.1
tmax = Tperi + period + period*0.1
tstep = (tmax-tmin)/tnum

tarr = findgen(tnum)*tstep + tmin

phase_mod = (tarr-Tperi)/period
;phase_mod = phase_mod - fix(phase_mod)
;for i=0, tnum-1 do if (phase_mod(i) lt 0.0) $
;  then phase_mod(i) = phase_mod(i) + 1.0

calc_sb2fit, EL_sb2, tarr, V1mod, V2mod

; Determine plotting ranges based on RV curves and measurements
vmax = max([max(V1mod),max(V2mod),V1_sb2])
vmin = min([min(V1mod),min(V2mod),V2_sb2])

; Extend plotting ranges by 10% 
; (be careful - this will be different depending whether RVs or
; positve or negative)
if (vmax ge 0) then vmax = vmax*1.1 else vmax = vmax*1.1
if (vmin lt 0) then vmin = vmin*1.1 else vmin = vmin*0.9

; Define user symbol to be rectangle
xrect = frac*[-1.0,-1.0,1.0,1.0,-1.0]
yrect = frac*[-1.0,1.0,1.0,-1.0,-1.0]
usersym,xrect,yrect

; Define user symbol to be plus sign
xplus = frac*[-1.0,1.0,0.0,0.0,0.0]
yplus = frac*[0.0,0.0,0.0,1.0,-1.0]
usersym,xplus,yplus

; Define user symbol to be open circle
; Make a vector of 16 points, Z[i] = 2pi/16:
Z = findgen(17) * (!pi*2/16.)
;Define the symbol to be a unit circle with 16 points
usersym, frac*cos(Z), frac*sin(Z)


set_plot, 'ps'
device, /Color, Bits_per_pixel=8, filename='temp.eps',xsize=17,ysize=15,/encap

plot,phase_sb2,V1_sb2,psym=4,yrange=[vmin,vmax],xrange =[-0.1,1.1], $
  xtitle = '!3Phase', ytitle = 'Velocity (km/s)', xstyle=1, ystyle=1, /NoData
usersym, frac*cos(Z), frac*sin(Z),thick=symthick,/fill
oplot,phase_sb2,V1_sb2,psym=8,color=3
usersym,xrect,yrect,thick=symthick,/fill
oplot,phase_sb2,V2_sb2,psym=8,color=1
usersym,xplus,yplus,thick=symthick
oplot,phase_sb1,V1_sb1,psym=8,color=3
oplot,phase_mod,V1mod,LineStyle=0.0,color=3
oplot,phase_mod,V2mod,Linestyle=0.0,color=1

plot_erry,phase_sb2,V1_sb2,dV1_sb2,sb2_num,3
plot_erry,phase_sb2,V2_sb2,dV2_sb2,sb2_num,1
plot_erry,phase_sb1,V1_sb1,dV1_sb1,sb1_num,3

oplot,[-0.1,1.1],[Vsys,Vsys],Linestyle=1

device, /close
set_plot, 'x'

print,'--------------------------------'

for i=0, sb2_num-1 do  print,time_sb2(i),V1_sb2(i),V2_sb2(i),phase_sb2(i)

print,'--------------------------------'

for i=0, sb1_num-1 do  print,time_sb1(i),V1_sb1(i),phase_sb1(i)

;; since model is plotted as solid line, it will suffice just to
;; connect data point directly to solid lne (without plotting fit positions)
;for l=0, sb2_num-1 do oplot,[phase_sb2(l),phase_sb2(l)],[V1_sb2(l),V1_f_sb2(l)]
;for l=0, sb2_num-1 do oplot,[phase_sb2(l),phase_sb2(l)],[V2_sb2(l),V2_f_sb2(l)]
;for l=0, sb1_num-1 do oplot,[phase_sb1(l),phase_sb1(l)],[V1_sb1(l),V1_f_sb1(l)]



; Plot residuals below RV curves, but make separate residual plots
; for RV1 and RV2 measurements

set_plot, 'ps'
device, /Color, Bits_per_pixel=8, filename='temp_res2.eps',xsize=12,ysize=16,/encap

;vmax = 220
;vmin = -110

;vmax = 5.0
;vmin = -5.0

;vmax = 19.0
;vmin = 8.0

;vmin=-50.0
;vmax=55.0

;vmin=-50.0
;vmax=55.0

plot,phase_sb2,V1_sb2,psym=4,yrange=[vmin,vmax],xrange =[-0.1,1.1], $
  ytitle = 'Radial Velocity (km/s)', xstyle=1,ystyle=1,/NoData, $
  position=[0.15,0.48,0.98,0.98],XTICKFORMAT="(A1)"

oplot,[-0.1,1.1],[Vsys,Vsys],Linestyle=1

if (num_zero gt 0) then begin
   usersym, frac*cos(Z), frac*sin(Z),thick=symthick,/fill
   oplot,phase_sb2_zero,V1_sb2_zero,psym=8
   usersym,xrect,yrect,thick=symthick,/fill
   oplot,phase_sb2_zero,V2_sb2_zero,psym=8
   calc_sb2fit, EL_sb2, time_sb2_zero, V1fit_zero, V2fit_zero
   diffV1_zero = V1_sb2_zero - V1fit_zero
   diffV2_zero = V2_sb2_zero - V2fit_zero
endif

usersym, frac*cos(Z), frac*sin(Z),thick=symthick
oplot,phase_sb1,V1_sb1,psym=8,thick=symthick


oplot,phase_mod,V1mod,LineStyle=0.0,thick=lthick,color=3
oplot,phase_mod,V2mod,Linestyle=0.0,thick=lthick,color=1

usersym, frac*cos(Z), frac*sin(Z),thick=symthick,/fill
oploterror,phase_sb2,V1_sb2,dV1_sb2,psym=8, errthick=1
oplot,phase_sb2,V1_sb2,psym=8,color=3

usersym,xrect,yrect,thick=symthick,/fill
oploterror,phase_sb2,V2_sb2,dV2_sb2,psym=8, errthick=1
oplot,phase_sb2,V2_sb2,psym=8,color=1

diffV1 = V1_sb2 - V1fit
diffV2 = V2_sb2 - V2fit

dmax1 = 1.1*max(abs([diffV1,dV1_sb2+diffV1,dV1_sb2-diffV1]))
dmax2 = 1.1*max(abs([diffV2,dV2_sb2+diffV2,dV2_sb2-diffV2]))
;dmax1 = 30.0
;dmax2 = 10.0
;dmax1 = 9.0
;dmax2 = 9.0
;dmax1 = 59.99
;dmax2 = 14.99
;dmax1 = 0.9
;dmax2 = 0.9
dmax1 = 0.99
dmax2 = 0.99

ploterror,phase_sb2,diffV1,dV1_sb2,psym=4,yrange=[-dmax1,dmax1],xrange =[-0.1,1.1], $
  ytitle = 'Residual (km/s)', xstyle=1, ystyle=1, errthick=1, /NoData, $
  position=[0.15,0.28,0.98,0.48],/noerase,XTICKFORMAT="(A1)"

usersym, frac*cos(Z), frac*sin(Z),thick=symthick,/fill
if (num_zero gt 0) then oplot,phase_sb2_zero,diffV1_zero,psym=8
oplot,phase_sb2,diffV1,psym=8,color=3

    ; red = 250, orange=220, yellow=195, green=150, blue=60, purple=30
    ; dark red = 260, aqua = 100
    ; ctab = [250,220,195,150,60,30,260,100]

oplot,[-0.1,1.1],[0,0],Linestyle=2,thick=lthick

ploterror,phase_sb2,diffV2,dV2_sb2,psym=4,yrange=[-dmax2,dmax2],xrange =[-0.1,1.1], $
  xtitle = 'Phase', ytitle = 'Residual (km/s)', xstyle=1, ystyle=1, errthick=1, /NoData,  $
  position=[0.15,0.08,0.98,0.28],/noerase

usersym,xrect,yrect,thick=symthick,/fill
if (num_zero gt 0) then oplot,phase_sb2_zero,diffV2_zero,psym=8
oplot,phase_sb2,diffV2,psym=8,color=1


oplot,[-0.1,1.1],[0,0],Linestyle=2,thick=lthick

device, /close
set_plot, 'x'




; Plot residuals below RV curves, but make separate residual plots
; for RV1 and RV2 measurements

set_plot, 'ps'
device, /Color, Bits_per_pixel=8, filename='temp_res3.eps',xsize=36,ysize=10,/encap

;vmax = 220
;vmin = -110

;vmax = 5.0
;vmin = -5.0

;vmax = 19.0
;vmin = 8.0

;vmin=-50.0
;vmax=55.0

;vmin=-50.0
;vmax=55.0

plot,phase_sb2,V1_sb2,psym=4,yrange=[vmin,vmax],xrange =[-0.1,1.1], $
  ytitle = 'Radial Velocity (km/s)', xstyle=1,ystyle=1,/NoData, $
  position=[0.04,0.15,0.32,0.98],xtitle = 'Phase'


oplot,[-0.1,1.1],[Vsys,Vsys],Linestyle=1

if (num_zero gt 0) then begin
   usersym, frac*cos(Z), frac*sin(Z),thick=symthick,/fill
   oplot,phase_sb2_zero,V1_sb2_zero,psym=8
   usersym,xrect,yrect,thick=symthick,/fill
   oplot,phase_sb2_zero,V2_sb2_zero,psym=8
   calc_sb2fit, EL_sb2, time_sb2_zero, V1fit_zero, V2fit_zero
   diffV1_zero = V1_sb2_zero - V1fit_zero
   diffV2_zero = V2_sb2_zero - V2fit_zero
endif

usersym, frac*cos(Z), frac*sin(Z),thick=symthick
oplot,phase_sb1,V1_sb1,psym=1,thick=4


oplot,phase_mod,V1mod,LineStyle=0.0,thick=lthick,color=3
oplot,phase_mod,V2mod,Linestyle=0.0,thick=lthick,color=1

usersym, frac*cos(Z), frac*sin(Z),thick=symthick,/fill
;oploterror,phase_sb2,V1_sb2,dV1_sb2,psym=8, errthick=1
oplot,phase_sb2,V1_sb2,psym=8,color=3

usersym,xrect,yrect,thick=symthick,/fill
;oploterror,phase_sb2,V2_sb2,dV2_sb2,psym=8, errthick=1
oplot,phase_sb2,V2_sb2,psym=8,color=1

;oplot,[-0.1,1.1],[0,0],Linestyle=2,thick=lthick

min_zoom = 0.95
max_zoom = 1.10

plot,phase_sb2,V1_sb2,psym=4,yrange=[vmin,vmax],xrange=[min_zoom,max_zoom], $
  xtitle = 'Phase',ytitle = 'Radial Velocity (km/s)', $
  xstyle=1,ystyle=1,/NoData, $
  position=[0.37,0.15,0.65,0.98],/noerase;,YTICKFORMAT="(A1)"


oplot,[-0.1,1.1],[Vsys,Vsys],Linestyle=1

if (num_zero gt 0) then begin
   usersym, frac*cos(Z), frac*sin(Z),thick=symthick,/fill
   oplot,phase_sb2_zero,V1_sb2_zero,psym=8
   oplot,phase_sb2_zero+1,V1_sb2_zero,psym=8
   usersym,xrect,yrect,thick=symthick,/fill
   oplot,phase_sb2_zero,V2_sb2_zero,psym=8
   oplot,phase_sb2_zero+1,V2_sb2_zero,psym=8
   calc_sb2fit, EL_sb2, time_sb2_zero, V1fit_zero, V2fit_zero
   diffV1_zero = V1_sb2_zero - V1fit_zero
   diffV2_zero = V2_sb2_zero - V2fit_zero
endif

usersym, frac*cos(Z), frac*sin(Z),thick=symthick
;oplot,phase_sb1,V1_sb1,psym=1


oplot,phase_mod,V1mod,LineStyle=0.0,thick=lthick,color=3
oplot,phase_mod,V2mod,Linestyle=0.0,thick=lthick,color=1

usersym, frac*cos(Z), frac*sin(Z),thick=symthick,/fill
;oploterror,phase_sb2,V1_sb2,dV1_sb2,psym=8, errthick=1
oplot,phase_sb2,V1_sb2,psym=8,color=3
oplot,phase_sb2+1,V1_sb2,psym=8,color=3

usersym,xrect,yrect,thick=symthick,/fill
;oploterror,phase_sb2,V2_sb2,dV2_sb2,psym=8, errthick=1
oplot,phase_sb2,V2_sb2,psym=8,color=1
oplot,phase_sb2+1,V2_sb2,psym=8,color=1


diffV1 = V1_sb2 - V1fit
diffV2 = V2_sb2 - V2fit

dmax1 = 1.1*max(abs([diffV1,dV1_sb2+diffV1,dV1_sb2-diffV1]))
dmax2 = 1.1*max(abs([diffV2,dV2_sb2+diffV2,dV2_sb2-diffV2]))
;dmax1 = 30.0
;dmax2 = 10.0
;dmax1 = 9.0
;dmax2 = 9.0
;dmax1 = 59.99
;dmax2 = 14.99
;dmax1 = 0.9
;dmax2 = 0.9
dmax1 = 0.99
dmax2 = 0.99




;plot,phase_sb2,V1_sb2,psym=4,yrange=[vmin,vmax],xrange =[-0.1,1.1], $
;  ytitle = 'Radial Velocity (km/s)', xstyle=1,ystyle=1,/NoData, $
;  position=[0.71,0.15,0.99,0.98],/noerase;,XTICKFORMAT="(A1)"

ploterror,phase_sb2,diffV1,dV1_sb2,psym=4,yrange=[-dmax1,dmax1],xrang =[min_zoom,max_zoom], $
  xtitle = 'Phase', ytitle = 'Residual (km/s)', xstyle=1, ystyle=1, errthick=1, /NoData, $
  position=[0.70,0.15,0.98,0.565],/noerase;,XTICKFORMAT="(A1)"

oploterror,phase_sb2+1,diffV1,dV1_sb2,psym=4,errthick=1,/NoData

usersym, frac*cos(Z), frac*sin(Z),thick=symthick,/fill
if (num_zero gt 0) then begin
   oplot,phase_sb2_zero,diffV1_zero,psym=8
   oplot,phase_sb2_zero+1,diffV1_zero,psym=8
endif
oplot,phase_sb2,diffV1,psym=8,color=3
oplot,phase_sb2+1,diffV1,psym=8,color=3

;    ; red = 250, orange=220, yellow=195, green=150, blue=60, purple=30
;    ; dark red = 260, aqua = 100
;    ; ctab = [250,220,195,150,60,30,260,100]

oplot,[-0.1,1.1],[0,0],Linestyle=2,thick=lthick

ploterror,phase_sb2,diffV2,dV2_sb2,psym=4,yrange=[-dmax2,dmax2],xrange=[min_zoom,max_zoom], $
  ytitle = 'Residual (km/s)', xstyle=1, ystyle=1, errthick=1, /NoData,  $
  position=[0.70,0.565,0.98,0.98],/noerase,XTICKFORMAT="(A1)"

oploterror,phase_sb2+1,diffV2,dV2_sb2,psym=4,errthick=1,/NoData

usersym,xrect,yrect,thick=symthick,/fill
if (num_zero gt 0) then begin
   oplot,phase_sb2_zero,diffV2_zero,psym=8
   oplot,phase_sb2_zero+1,diffV2_zero,psym=8
endif
oplot,phase_sb2,diffV2,psym=8,color=1
oplot,phase_sb2+1,diffV2,psym=8,color=1


oplot,[-0.1,1.1],[0,0],Linestyle=2,thick=lthick

device, /close
set_plot, 'x'

free_lun,lun

end

;
;
;

pro plot_erry, x, y, dy, N, color_ind

; plot error bars associated with each data point
; Input
;	x: array of x values
;	dx: array of error in x values
;	y: array of y values
;	dy: erray of error in y values
;	N: number of data points
;       color_ind: color index

for i=0, N-1 do begin

;	oplot, [x(i)-dx(i), x(i)+dx(i)], [y(i), y(i)]
	oplot, [x(i), x(i)], [y(i)-dy(i), y(i)+dy(i)],color=color_ind

endfor

end
